# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.tier import Tier
from ..shared.discount import Discount
from ..shared.pro_service import ProService
from ..shared.subscription import Subscription
from ..shared.override_tier import OverrideTier
from ..shared.commit_specifier import CommitSpecifier
from ..shared.credit_type_data import CreditTypeData
from ..shared.schedule_duration import ScheduleDuration
from ..shared.commit_specifier_input import CommitSpecifierInput
from ..shared.payment_gate_config_v2 import PaymentGateConfigV2
from ..shared.schedule_point_in_time import SchedulePointInTime
from ..shared.update_base_threshold_commit import UpdateBaseThresholdCommit
from ..shared.commit_hierarchy_configuration import CommitHierarchyConfiguration
from ..shared.spend_threshold_configuration_v2 import SpendThresholdConfigurationV2
from ..shared.recurring_commit_subscription_config import RecurringCommitSubscriptionConfig
from ..shared.prepaid_balance_threshold_configuration_v2 import PrepaidBalanceThresholdConfigurationV2

__all__ = [
    "ContractGetEditHistoryResponse",
    "Data",
    "DataAddCommit",
    "DataAddCommitProduct",
    "DataAddCommitInvoiceSchedule",
    "DataAddCommitInvoiceScheduleScheduleItem",
    "DataAddCredit",
    "DataAddCreditProduct",
    "DataAddOverride",
    "DataAddOverrideOverrideSpecifier",
    "DataAddOverrideOverwriteRate",
    "DataAddOverrideProduct",
    "DataAddRecurringCommit",
    "DataAddRecurringCommitAccessAmount",
    "DataAddRecurringCommitCommitDuration",
    "DataAddRecurringCommitProduct",
    "DataAddRecurringCommitContract",
    "DataAddRecurringCommitInvoiceAmount",
    "DataAddRecurringCredit",
    "DataAddRecurringCreditAccessAmount",
    "DataAddRecurringCreditCommitDuration",
    "DataAddRecurringCreditProduct",
    "DataAddRecurringCreditContract",
    "DataAddResellerRoyalty",
    "DataAddScheduledCharge",
    "DataAddScheduledChargeProduct",
    "DataAddUsageFilter",
    "DataArchiveCommit",
    "DataArchiveCredit",
    "DataArchiveScheduledCharge",
    "DataRemoveOverride",
    "DataUpdateCommit",
    "DataUpdateCommitAccessSchedule",
    "DataUpdateCommitAccessScheduleAddScheduleItem",
    "DataUpdateCommitAccessScheduleRemoveScheduleItem",
    "DataUpdateCommitAccessScheduleUpdateScheduleItem",
    "DataUpdateCommitInvoiceSchedule",
    "DataUpdateCommitInvoiceScheduleAddScheduleItem",
    "DataUpdateCommitInvoiceScheduleRemoveScheduleItem",
    "DataUpdateCommitInvoiceScheduleUpdateScheduleItem",
    "DataUpdateCredit",
    "DataUpdateCreditAccessSchedule",
    "DataUpdateCreditAccessScheduleAddScheduleItem",
    "DataUpdateCreditAccessScheduleRemoveScheduleItem",
    "DataUpdateCreditAccessScheduleUpdateScheduleItem",
    "DataUpdateDiscount",
    "DataUpdateDiscountSchedule",
    "DataUpdateDiscountScheduleRecurringSchedule",
    "DataUpdateDiscountScheduleScheduleItem",
    "DataUpdatePrepaidBalanceThresholdConfiguration",
    "DataUpdatePrepaidBalanceThresholdConfigurationCommit",
    "DataUpdateRecurringCommit",
    "DataUpdateRecurringCommitAccessAmount",
    "DataUpdateRecurringCommitInvoiceAmount",
    "DataUpdateRecurringCredit",
    "DataUpdateRecurringCreditAccessAmount",
    "DataUpdateRefundInvoice",
    "DataUpdateScheduledCharge",
    "DataUpdateScheduledChargeInvoiceSchedule",
    "DataUpdateScheduledChargeInvoiceScheduleAddScheduleItem",
    "DataUpdateScheduledChargeInvoiceScheduleRemoveScheduleItem",
    "DataUpdateScheduledChargeInvoiceScheduleUpdateScheduleItem",
    "DataUpdateSpendThresholdConfiguration",
    "DataUpdateSubscription",
    "DataUpdateSubscriptionQuantityUpdate",
    "DataUpdateSubscriptionSeatUpdates",
    "DataUpdateSubscriptionSeatUpdatesAddSeatID",
    "DataUpdateSubscriptionSeatUpdatesAddUnassignedSeat",
    "DataUpdateSubscriptionSeatUpdatesRemoveSeatID",
    "DataUpdateSubscriptionSeatUpdatesRemoveUnassignedSeat",
]


class DataAddCommitProduct(BaseModel):
    id: str

    name: str


class DataAddCommitInvoiceScheduleScheduleItem(BaseModel):
    id: str

    timestamp: datetime

    amount: Optional[float] = None

    invoice_id: Optional[str] = None

    quantity: Optional[float] = None

    unit_price: Optional[float] = None


class DataAddCommitInvoiceSchedule(BaseModel):
    """The schedule that the customer will be invoiced for this commit."""

    credit_type: Optional[CreditTypeData] = None

    do_not_invoice: Optional[bool] = None
    """If true, this schedule will not generate an invoice."""

    schedule_items: Optional[List[DataAddCommitInvoiceScheduleScheduleItem]] = None


class DataAddCommit(BaseModel):
    id: str

    product: DataAddCommitProduct

    type: Literal["PREPAID", "POSTPAID"]

    access_schedule: Optional[ScheduleDuration] = None
    """
    The schedule that the customer will gain access to the credits purposed with
    this commit.
    """

    applicable_product_ids: Optional[List[str]] = None

    applicable_product_tags: Optional[List[str]] = None

    description: Optional[str] = None

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for commit hierarchy access control"""

    invoice_schedule: Optional[DataAddCommitInvoiceSchedule] = None
    """The schedule that the customer will be invoiced for this commit."""

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    priority: Optional[float] = None
    """
    If multiple credits or commits are applicable, the one with the lower priority
    will apply first.
    """

    rate_type: Optional[Literal["COMMIT_RATE", "LIST_RATE"]] = None

    rollover_fraction: Optional[float] = None

    salesforce_opportunity_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    specifiers: Optional[List[CommitSpecifierInput]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    Instead, to target usage by product or product tag, pass those values in the
    body of `specifiers`.
    """


class DataAddCreditProduct(BaseModel):
    id: str

    name: str


class DataAddCredit(BaseModel):
    id: str

    product: DataAddCreditProduct

    type: Literal["CREDIT"]

    access_schedule: Optional[ScheduleDuration] = None
    """The schedule that the customer will gain access to the credits."""

    applicable_product_ids: Optional[List[str]] = None

    applicable_product_tags: Optional[List[str]] = None

    description: Optional[str] = None

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for recurring credit hierarchy access control"""

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    priority: Optional[float] = None
    """
    If multiple credits or commits are applicable, the one with the lower priority
    will apply first.
    """

    salesforce_opportunity_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    specifiers: Optional[List[CommitSpecifierInput]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    Instead, to target usage by product or product tag, pass those values in the
    body of `specifiers`.
    """


class DataAddOverrideOverrideSpecifier(BaseModel):
    billing_frequency: Optional[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]] = None

    commit_ids: Optional[List[str]] = None

    presentation_group_values: Optional[Dict[str, Optional[str]]] = None

    pricing_group_values: Optional[Dict[str, str]] = None

    product_id: Optional[str] = None

    product_tags: Optional[List[str]] = None

    recurring_commit_ids: Optional[List[str]] = None

    recurring_credit_ids: Optional[List[str]] = None


class DataAddOverrideOverwriteRate(BaseModel):
    rate_type: Literal["FLAT", "PERCENTAGE", "SUBSCRIPTION", "TIERED", "CUSTOM"]

    credit_type: Optional[CreditTypeData] = None

    custom_rate: Optional[Dict[str, object]] = None
    """Only set for CUSTOM rate_type.

    This field is interpreted by custom rate processors.
    """

    is_prorated: Optional[bool] = None
    """Default proration configuration.

    Only valid for SUBSCRIPTION rate_type. Must be set to true.
    """

    price: Optional[float] = None
    """Default price.

    For FLAT rate_type, this must be >=0. For PERCENTAGE rate_type, this is a
    decimal fraction, e.g. use 0.1 for 10%; this must be >=0 and <=1.
    """

    quantity: Optional[float] = None
    """Default quantity. For SUBSCRIPTION rate_type, this must be >=0."""

    tiers: Optional[List[Tier]] = None
    """Only set for TIERED rate_type."""


class DataAddOverrideProduct(BaseModel):
    id: str

    name: str


class DataAddOverride(BaseModel):
    id: str

    starting_at: datetime

    applicable_product_tags: Optional[List[str]] = None

    ending_before: Optional[datetime] = None

    entitled: Optional[bool] = None

    is_commit_specific: Optional[bool] = None

    multiplier: Optional[float] = None

    override_specifiers: Optional[List[DataAddOverrideOverrideSpecifier]] = None

    override_tiers: Optional[List[OverrideTier]] = None

    overwrite_rate: Optional[DataAddOverrideOverwriteRate] = None

    priority: Optional[float] = None

    product: Optional[DataAddOverrideProduct] = None

    target: Optional[Literal["COMMIT_RATE", "LIST_RATE"]] = None

    type: Optional[Literal["OVERWRITE", "MULTIPLIER", "TIERED"]] = None


class DataAddRecurringCommitAccessAmount(BaseModel):
    """The amount of commit to grant."""

    credit_type_id: str

    unit_price: float

    quantity: Optional[float] = None


class DataAddRecurringCommitCommitDuration(BaseModel):
    """The amount of time the created commits will be valid for"""

    value: float

    unit: Optional[Literal["PERIODS"]] = None


class DataAddRecurringCommitProduct(BaseModel):
    id: str

    name: str


class DataAddRecurringCommitContract(BaseModel):
    id: str


class DataAddRecurringCommitInvoiceAmount(BaseModel):
    """The amount the customer should be billed for the commit. Not required."""

    credit_type_id: str

    quantity: float

    unit_price: float


class DataAddRecurringCommit(BaseModel):
    id: str

    access_amount: DataAddRecurringCommitAccessAmount
    """The amount of commit to grant."""

    commit_duration: DataAddRecurringCommitCommitDuration
    """The amount of time the created commits will be valid for"""

    priority: float
    """Will be passed down to the individual commits"""

    product: DataAddRecurringCommitProduct

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]
    """Whether the created commits will use the commit rate or list rate"""

    starting_at: datetime
    """Determines the start time for the first commit"""

    applicable_product_ids: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    applicable_product_tags: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    contract: Optional[DataAddRecurringCommitContract] = None

    description: Optional[str] = None
    """Will be passed down to the individual commits"""

    ending_before: Optional[datetime] = None
    """Determines when the contract will stop creating recurring commits. Optional"""

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for recurring credit hierarchy access control"""

    invoice_amount: Optional[DataAddRecurringCommitInvoiceAmount] = None
    """The amount the customer should be billed for the commit. Not required."""

    name: Optional[str] = None
    """Displayed on invoices. Will be passed through to the individual commits"""

    netsuite_sales_order_id: Optional[str] = None
    """Will be passed down to the individual commits"""

    proration: Optional[Literal["NONE", "FIRST", "LAST", "FIRST_AND_LAST"]] = None
    """Determines whether the first and last commit will be prorated.

    If not provided, the default is FIRST_AND_LAST (i.e. prorate both the first and
    last commits).
    """

    recurrence_frequency: Optional[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]] = None
    """The frequency at which the recurring commits will be created.

    If not provided: - The commits will be created on the usage invoice frequency.
    If provided: - The period defined in the duration will correspond to this
    frequency. - Commits will be created aligned with the recurring commit's
    starting_at rather than the usage invoice dates.
    """

    rollover_fraction: Optional[float] = None
    """Will be passed down to the individual commits.

    This controls how much of an individual unexpired commit will roll over upon
    contract transition. Must be between 0 and 1.
    """

    specifiers: Optional[List[CommitSpecifier]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown.
    """

    subscription_config: Optional[RecurringCommitSubscriptionConfig] = None
    """Attach a subscription to the recurring commit/credit."""


class DataAddRecurringCreditAccessAmount(BaseModel):
    """The amount of commit to grant."""

    credit_type_id: str

    unit_price: float

    quantity: Optional[float] = None


class DataAddRecurringCreditCommitDuration(BaseModel):
    """The amount of time the created commits will be valid for"""

    value: float

    unit: Optional[Literal["PERIODS"]] = None


class DataAddRecurringCreditProduct(BaseModel):
    id: str

    name: str


class DataAddRecurringCreditContract(BaseModel):
    id: str


class DataAddRecurringCredit(BaseModel):
    id: str

    access_amount: DataAddRecurringCreditAccessAmount
    """The amount of commit to grant."""

    commit_duration: DataAddRecurringCreditCommitDuration
    """The amount of time the created commits will be valid for"""

    priority: float
    """Will be passed down to the individual commits"""

    product: DataAddRecurringCreditProduct

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]
    """Whether the created commits will use the commit rate or list rate"""

    starting_at: datetime
    """Determines the start time for the first commit"""

    applicable_product_ids: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    applicable_product_tags: Optional[List[str]] = None
    """Will be passed down to the individual commits"""

    contract: Optional[DataAddRecurringCreditContract] = None

    description: Optional[str] = None
    """Will be passed down to the individual commits"""

    ending_before: Optional[datetime] = None
    """Determines when the contract will stop creating recurring commits. Optional"""

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for recurring credit hierarchy access control"""

    name: Optional[str] = None
    """Displayed on invoices. Will be passed through to the individual commits"""

    netsuite_sales_order_id: Optional[str] = None
    """Will be passed down to the individual commits"""

    proration: Optional[Literal["NONE", "FIRST", "LAST", "FIRST_AND_LAST"]] = None
    """Determines whether the first and last commit will be prorated.

    If not provided, the default is FIRST_AND_LAST (i.e. prorate both the first and
    last commits).
    """

    recurrence_frequency: Optional[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]] = None
    """The frequency at which the recurring commits will be created.

    If not provided: - The commits will be created on the usage invoice frequency.
    If provided: - The period defined in the duration will correspond to this
    frequency. - Commits will be created aligned with the recurring commit's
    starting_at rather than the usage invoice dates.
    """

    rollover_fraction: Optional[float] = None
    """Will be passed down to the individual commits.

    This controls how much of an individual unexpired commit will roll over upon
    contract transition. Must be between 0 and 1.
    """

    specifiers: Optional[List[CommitSpecifier]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown.
    """

    subscription_config: Optional[RecurringCommitSubscriptionConfig] = None
    """Attach a subscription to the recurring commit/credit."""


class DataAddResellerRoyalty(BaseModel):
    reseller_type: Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]

    applicable_product_ids: Optional[List[str]] = None

    applicable_product_tags: Optional[List[str]] = None

    aws_account_number: Optional[str] = None

    aws_offer_id: Optional[str] = None

    aws_payer_reference_id: Optional[str] = None

    ending_before: Optional[datetime] = None

    fraction: Optional[float] = None

    gcp_account_id: Optional[str] = None

    gcp_offer_id: Optional[str] = None

    netsuite_reseller_id: Optional[str] = None

    reseller_contract_value: Optional[float] = None

    starting_at: Optional[datetime] = None


class DataAddScheduledChargeProduct(BaseModel):
    id: str

    name: str


class DataAddScheduledCharge(BaseModel):
    id: str

    product: DataAddScheduledChargeProduct

    schedule: SchedulePointInTime

    name: Optional[str] = None
    """displayed on invoices"""

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""


class DataAddUsageFilter(BaseModel):
    group_key: str

    group_values: List[str]

    starting_at: datetime
    """
    This will match contract starting_at value if usage filter is active from the
    beginning of the contract.
    """

    ending_before: Optional[datetime] = None
    """
    This will match contract ending_before value if usage filter is active until the
    end of the contract. It will be undefined if the contract is open-ended.
    """


class DataArchiveCommit(BaseModel):
    id: str


class DataArchiveCredit(BaseModel):
    id: str


class DataArchiveScheduledCharge(BaseModel):
    id: str


class DataRemoveOverride(BaseModel):
    id: str


class DataUpdateCommitAccessScheduleAddScheduleItem(BaseModel):
    amount: float

    ending_before: datetime
    """RFC 3339 timestamp (exclusive)"""

    starting_at: datetime
    """RFC 3339 timestamp (inclusive)"""


class DataUpdateCommitAccessScheduleRemoveScheduleItem(BaseModel):
    id: str


class DataUpdateCommitAccessScheduleUpdateScheduleItem(BaseModel):
    id: str

    amount: Optional[float] = None

    ending_before: Optional[datetime] = None
    """RFC 3339 timestamp (exclusive)"""

    starting_at: Optional[datetime] = None
    """RFC 3339 timestamp (inclusive)"""


class DataUpdateCommitAccessSchedule(BaseModel):
    add_schedule_items: Optional[List[DataUpdateCommitAccessScheduleAddScheduleItem]] = None

    remove_schedule_items: Optional[List[DataUpdateCommitAccessScheduleRemoveScheduleItem]] = None

    update_schedule_items: Optional[List[DataUpdateCommitAccessScheduleUpdateScheduleItem]] = None


class DataUpdateCommitInvoiceScheduleAddScheduleItem(BaseModel):
    timestamp: datetime

    amount: Optional[float] = None

    quantity: Optional[float] = None

    unit_price: Optional[float] = None


class DataUpdateCommitInvoiceScheduleRemoveScheduleItem(BaseModel):
    id: str


class DataUpdateCommitInvoiceScheduleUpdateScheduleItem(BaseModel):
    id: str

    amount: Optional[float] = None

    quantity: Optional[float] = None

    timestamp: Optional[datetime] = None

    unit_price: Optional[float] = None


class DataUpdateCommitInvoiceSchedule(BaseModel):
    add_schedule_items: Optional[List[DataUpdateCommitInvoiceScheduleAddScheduleItem]] = None

    remove_schedule_items: Optional[List[DataUpdateCommitInvoiceScheduleRemoveScheduleItem]] = None

    update_schedule_items: Optional[List[DataUpdateCommitInvoiceScheduleUpdateScheduleItem]] = None


class DataUpdateCommit(BaseModel):
    id: str

    access_schedule: Optional[DataUpdateCommitAccessSchedule] = None

    applicable_product_ids: Optional[List[str]] = None
    """Which products the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    applicable_product_tags: Optional[List[str]] = None
    """Which tags the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    description: Optional[str] = None

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for commit hierarchy access control"""

    invoice_schedule: Optional[DataUpdateCommitInvoiceSchedule] = None

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None

    priority: Optional[float] = None
    """
    If multiple commits are applicable, the one with the lower priority will apply
    first.
    """

    product_id: Optional[str] = None

    rate_type: Optional[Literal["COMMIT_RATE", "LIST_RATE"]] = None
    """If set, the commit's rate type was updated to the specified value."""

    rollover_fraction: Optional[float] = None

    specifiers: Optional[List[CommitSpecifierInput]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    Instead, to target usage by product or product tag, pass those values in the
    body of `specifiers`.
    """


class DataUpdateCreditAccessScheduleAddScheduleItem(BaseModel):
    amount: float

    ending_before: datetime
    """RFC 3339 timestamp (exclusive)"""

    starting_at: datetime
    """RFC 3339 timestamp (inclusive)"""


class DataUpdateCreditAccessScheduleRemoveScheduleItem(BaseModel):
    id: str


class DataUpdateCreditAccessScheduleUpdateScheduleItem(BaseModel):
    id: str

    amount: Optional[float] = None

    ending_before: Optional[datetime] = None
    """RFC 3339 timestamp (exclusive)"""

    starting_at: Optional[datetime] = None
    """RFC 3339 timestamp (inclusive)"""


class DataUpdateCreditAccessSchedule(BaseModel):
    add_schedule_items: Optional[List[DataUpdateCreditAccessScheduleAddScheduleItem]] = None

    remove_schedule_items: Optional[List[DataUpdateCreditAccessScheduleRemoveScheduleItem]] = None

    update_schedule_items: Optional[List[DataUpdateCreditAccessScheduleUpdateScheduleItem]] = None


class DataUpdateCredit(BaseModel):
    id: str

    access_schedule: Optional[DataUpdateCreditAccessSchedule] = None

    description: Optional[str] = None

    hierarchy_configuration: Optional[CommitHierarchyConfiguration] = None
    """Optional configuration for credit hierarchy access control"""

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None

    priority: Optional[float] = None
    """
    If multiple credits are applicable, the one with the lower priority will apply
    first.
    """

    rate_type: Optional[Literal["LIST_RATE", "COMMIT_RATE"]] = None
    """If set, the credit's rate type was updated to the specified value."""

    rollover_fraction: Optional[float] = None


class DataUpdateDiscountScheduleRecurringSchedule(BaseModel):
    """Enter the unit price and quantity for the charge or instead only send the amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is inferred to be 1.
    """

    amount_distribution: Literal["DIVIDED", "DIVIDED_ROUNDED", "EACH"]

    ending_before: datetime
    """RFC 3339 timestamp (exclusive)."""

    frequency: Literal["MONTHLY", "QUARTERLY", "SEMI_ANNUAL", "ANNUAL", "WEEKLY"]

    starting_at: datetime
    """RFC 3339 timestamp (inclusive)."""

    amount: Optional[float] = None
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: Optional[float] = None
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: Optional[float] = None
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class DataUpdateDiscountScheduleScheduleItem(BaseModel):
    timestamp: datetime
    """timestamp of the scheduled event"""

    amount: Optional[float] = None
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: Optional[float] = None
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: Optional[float] = None
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class DataUpdateDiscountSchedule(BaseModel):
    """Must provide either schedule_items or recurring_schedule."""

    credit_type_id: Optional[str] = None
    """Defaults to USD (cents) if not passed."""

    do_not_invoice: Optional[bool] = None
    """This field is only applicable to commit invoice schedules.

    If true, this schedule will not generate an invoice.
    """

    recurring_schedule: Optional[DataUpdateDiscountScheduleRecurringSchedule] = None
    """Enter the unit price and quantity for the charge or instead only send the
    amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is
    inferred to be 1.
    """

    schedule_items: Optional[List[DataUpdateDiscountScheduleScheduleItem]] = None
    """Either provide amount or provide both unit_price and quantity."""


class DataUpdateDiscount(BaseModel):
    id: str

    custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None

    schedule: Optional[DataUpdateDiscountSchedule] = None
    """Must provide either schedule_items or recurring_schedule."""


class DataUpdatePrepaidBalanceThresholdConfigurationCommit(UpdateBaseThresholdCommit):
    applicable_product_ids: Optional[List[str]] = None
    """Which products the threshold commit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    commit applies to all products.
    """

    applicable_product_tags: Optional[List[str]] = None
    """Which tags the threshold commit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    commit applies to all products.
    """

    specifiers: Optional[List[CommitSpecifierInput]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    Instead, to target usage by product or product tag, pass those values in the
    body of `specifiers`.
    """


class DataUpdatePrepaidBalanceThresholdConfiguration(BaseModel):
    commit: Optional[DataUpdatePrepaidBalanceThresholdConfigurationCommit] = None

    custom_credit_type_id: Optional[str] = None
    """
    If provided, the threshold, recharge-to amount, and the resulting threshold
    commit amount will be in terms of this credit type instead of the fiat currency.
    """

    is_enabled: Optional[bool] = None
    """
    When set to false, the contract will not be evaluated against the
    threshold_amount. Toggling to true will result an immediate evaluation,
    regardless of prior state.
    """

    payment_gate_config: Optional[PaymentGateConfigV2] = None

    recharge_to_amount: Optional[float] = None
    """Specify the amount the balance should be recharged to."""

    threshold_amount: Optional[float] = None
    """Specify the threshold amount for the contract.

    Each time the contract's balance lowers to this amount, a threshold charge will
    be initiated.
    """


class DataUpdateRecurringCommitAccessAmount(BaseModel):
    quantity: Optional[float] = None

    unit_price: Optional[float] = None


class DataUpdateRecurringCommitInvoiceAmount(BaseModel):
    quantity: Optional[float] = None

    unit_price: Optional[float] = None


class DataUpdateRecurringCommit(BaseModel):
    id: str

    access_amount: Optional[DataUpdateRecurringCommitAccessAmount] = None

    ending_before: Optional[datetime] = None

    invoice_amount: Optional[DataUpdateRecurringCommitInvoiceAmount] = None

    rate_type: Optional[Literal["LIST_RATE", "COMMIT_RATE"]] = None


class DataUpdateRecurringCreditAccessAmount(BaseModel):
    quantity: Optional[float] = None

    unit_price: Optional[float] = None


class DataUpdateRecurringCredit(BaseModel):
    id: str

    access_amount: Optional[DataUpdateRecurringCreditAccessAmount] = None

    ending_before: Optional[datetime] = None

    rate_type: Optional[Literal["LIST_RATE", "COMMIT_RATE"]] = None


class DataUpdateRefundInvoice(BaseModel):
    date: datetime

    invoice_id: str


class DataUpdateScheduledChargeInvoiceScheduleAddScheduleItem(BaseModel):
    timestamp: datetime

    amount: Optional[float] = None

    quantity: Optional[float] = None

    unit_price: Optional[float] = None


class DataUpdateScheduledChargeInvoiceScheduleRemoveScheduleItem(BaseModel):
    id: str


class DataUpdateScheduledChargeInvoiceScheduleUpdateScheduleItem(BaseModel):
    id: str

    amount: Optional[float] = None

    quantity: Optional[float] = None

    timestamp: Optional[datetime] = None

    unit_price: Optional[float] = None


class DataUpdateScheduledChargeInvoiceSchedule(BaseModel):
    add_schedule_items: Optional[List[DataUpdateScheduledChargeInvoiceScheduleAddScheduleItem]] = None

    remove_schedule_items: Optional[List[DataUpdateScheduledChargeInvoiceScheduleRemoveScheduleItem]] = None

    update_schedule_items: Optional[List[DataUpdateScheduledChargeInvoiceScheduleUpdateScheduleItem]] = None


class DataUpdateScheduledCharge(BaseModel):
    id: str

    invoice_schedule: Optional[DataUpdateScheduledChargeInvoiceSchedule] = None

    name: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None


class DataUpdateSpendThresholdConfiguration(BaseModel):
    commit: Optional[UpdateBaseThresholdCommit] = None

    is_enabled: Optional[bool] = None
    """
    When set to false, the contract will not be evaluated against the
    threshold_amount. Toggling to true will result an immediate evaluation,
    regardless of prior state.
    """

    payment_gate_config: Optional[PaymentGateConfigV2] = None

    threshold_amount: Optional[float] = None
    """Specify the threshold amount for the contract.

    Each time the contract's usage hits this amount, a threshold charge will be
    initiated.
    """


class DataUpdateSubscriptionQuantityUpdate(BaseModel):
    starting_at: datetime

    quantity: Optional[float] = None

    quantity_delta: Optional[float] = None


class DataUpdateSubscriptionSeatUpdatesAddSeatID(BaseModel):
    seat_ids: List[str]

    starting_at: datetime
    """Assigned seats will be added/removed starting at this date."""


class DataUpdateSubscriptionSeatUpdatesAddUnassignedSeat(BaseModel):
    quantity: float
    """
    The number of unassigned seats on the subscription will increase/decrease by
    this delta. Must be greater than 0.
    """

    starting_at: datetime
    """Unassigned seats will be updated starting at this date."""


class DataUpdateSubscriptionSeatUpdatesRemoveSeatID(BaseModel):
    seat_ids: List[str]

    starting_at: datetime
    """Assigned seats will be added/removed starting at this date."""


class DataUpdateSubscriptionSeatUpdatesRemoveUnassignedSeat(BaseModel):
    quantity: float
    """
    The number of unassigned seats on the subscription will increase/decrease by
    this delta. Must be greater than 0.
    """

    starting_at: datetime
    """Unassigned seats will be updated starting at this date."""


class DataUpdateSubscriptionSeatUpdates(BaseModel):
    """Manage subscription seats for subscriptions in SEAT_BASED mode."""

    add_seat_ids: Optional[List[DataUpdateSubscriptionSeatUpdatesAddSeatID]] = None
    """Adds seat IDs to the subscription.

    If there are unassigned seats, the new seat IDs will fill these unassigned seats
    and not increase the total subscription quantity. Otherwise, if there are more
    new seat IDs than unassigned seats, the total subscription quantity will
    increase.
    """

    add_unassigned_seats: Optional[List[DataUpdateSubscriptionSeatUpdatesAddUnassignedSeat]] = None
    """Adds unassigned seats to the subscription.

    This will increase the total subscription quantity.
    """

    remove_seat_ids: Optional[List[DataUpdateSubscriptionSeatUpdatesRemoveSeatID]] = None
    """Removes seat IDs from the subscription, if possible.

    If a seat ID is removed, the total subscription quantity will decrease.
    Otherwise, if the seat ID is not found on the subscription, this is a no-op.
    """

    remove_unassigned_seats: Optional[List[DataUpdateSubscriptionSeatUpdatesRemoveUnassignedSeat]] = None
    """Removes unassigned seats from the subscription.

    This will decrease the total subscription quantity if there are are unassigned
    seats.
    """


class DataUpdateSubscription(BaseModel):
    id: str

    ending_before: Optional[datetime] = None

    quantity_updates: Optional[List[DataUpdateSubscriptionQuantityUpdate]] = None

    seat_updates: Optional[DataUpdateSubscriptionSeatUpdates] = None
    """Manage subscription seats for subscriptions in SEAT_BASED mode."""


class Data(BaseModel):
    id: str

    add_commits: Optional[List[DataAddCommit]] = None

    add_credits: Optional[List[DataAddCredit]] = None

    add_discounts: Optional[List[Discount]] = None

    add_overrides: Optional[List[DataAddOverride]] = None

    add_prepaid_balance_threshold_configuration: Optional[PrepaidBalanceThresholdConfigurationV2] = None

    add_pro_services: Optional[List[ProService]] = None

    add_recurring_commits: Optional[List[DataAddRecurringCommit]] = None

    add_recurring_credits: Optional[List[DataAddRecurringCredit]] = None

    add_reseller_royalties: Optional[List[DataAddResellerRoyalty]] = None

    add_scheduled_charges: Optional[List[DataAddScheduledCharge]] = None

    add_spend_threshold_configuration: Optional[SpendThresholdConfigurationV2] = None

    add_subscriptions: Optional[List[Subscription]] = None
    """List of subscriptions on the contract."""

    add_usage_filters: Optional[List[DataAddUsageFilter]] = None

    archive_commits: Optional[List[DataArchiveCommit]] = None

    archive_credits: Optional[List[DataArchiveCredit]] = None

    archive_scheduled_charges: Optional[List[DataArchiveScheduledCharge]] = None

    remove_overrides: Optional[List[DataRemoveOverride]] = None

    timestamp: Optional[datetime] = None

    uniqueness_key: Optional[str] = None
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """

    update_commits: Optional[List[DataUpdateCommit]] = None

    update_contract_end_date: Optional[datetime] = None

    update_contract_name: Optional[str] = None
    """Value to update the contract name to.

    If not provided, the contract name will remain unchanged.
    """

    update_credits: Optional[List[DataUpdateCredit]] = None

    update_discounts: Optional[List[DataUpdateDiscount]] = None

    update_prepaid_balance_threshold_configuration: Optional[DataUpdatePrepaidBalanceThresholdConfiguration] = None

    update_recurring_commits: Optional[List[DataUpdateRecurringCommit]] = None

    update_recurring_credits: Optional[List[DataUpdateRecurringCredit]] = None

    update_refund_invoices: Optional[List[DataUpdateRefundInvoice]] = None

    update_scheduled_charges: Optional[List[DataUpdateScheduledCharge]] = None

    update_spend_threshold_configuration: Optional[DataUpdateSpendThresholdConfiguration] = None

    update_subscriptions: Optional[List[DataUpdateSubscription]] = None
    """Optional list of subscriptions to update."""


class ContractGetEditHistoryResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContractRetrieveParams"]


class ContractRetrieveParams(TypedDict, total=False):
    contract_id: Required[str]

    customer_id: Required[str]

    as_of_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Optional RFC 3339 timestamp.

    Return the contract as of this date. Cannot be used with include_ledgers
    parameter.
    """

    include_balance: bool
    """Include the balance of credits and commits in the response.

    Setting this flag may cause the query to be slower.
    """

    include_ledgers: bool
    """Include commit/credit ledgers in the response.

    Setting this flag may cause the query to be slower. Cannot be used with
    as_of_date parameter.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import alert_create_params, alert_archive_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.alert_create_response import AlertCreateResponse
from ...types.v1.alert_archive_response import AlertArchiveResponse

__all__ = ["AlertsResource", "AsyncAlertsResource"]


class AlertsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AlertsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AlertsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AlertsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AlertsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        alert_type: Literal[
            "low_credit_balance_reached",
            "spend_threshold_reached",
            "monthly_invoice_total_spend_threshold_reached",
            "low_remaining_days_in_plan_reached",
            "low_remaining_credit_percentage_reached",
            "usage_threshold_reached",
            "low_remaining_days_for_commit_segment_reached",
            "low_remaining_commit_balance_reached",
            "low_remaining_commit_percentage_reached",
            "low_remaining_days_for_contract_credit_segment_reached",
            "low_remaining_contract_credit_balance_reached",
            "low_remaining_contract_credit_percentage_reached",
            "low_remaining_contract_credit_and_commit_balance_reached",
            "invoice_total_reached",
            "low_remaining_seat_balance_reached",
        ],
        name: str,
        threshold: float,
        billable_metric_id: str | Omit = omit,
        credit_grant_type_filters: SequenceNotStr[str] | Omit = omit,
        credit_type_id: str | Omit = omit,
        custom_field_filters: Iterable[alert_create_params.CustomFieldFilter] | Omit = omit,
        customer_id: str | Omit = omit,
        evaluate_on_create: bool | Omit = omit,
        group_values: Iterable[alert_create_params.GroupValue] | Omit = omit,
        invoice_types_filter: SequenceNotStr[str] | Omit = omit,
        plan_id: str | Omit = omit,
        seat_filter: alert_create_params.SeatFilter | Omit = omit,
        uniqueness_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlertCreateResponse:
        """
        Create a new threshold notification to monitor customer spending, balances, and
        billing metrics in real-time. Metronome's notification system provides
        industry-leading speed with immediate evaluation capabilities, enabling you to
        proactively manage customer accounts and prevent revenue leakage.

        This endpoint creates configurable threshold notifications that continuously
        monitor various billing thresholds including spend limits, credit balances,
        commitment utilization, and invoice totals. Threshold notifications can be
        configured globally for all customers or targeted to specific customer accounts.

        ### Use this endpoint to:

        - Proactively monitor customer spending patterns to prevent unexpected overages
          or credit exhaustion
        - Automate notifications when customers approach commitment limits or credit
          thresholds
        - Enable real-time intervention for accounts at risk of churn or payment issues
        - Scale billing operations by automating threshold-based workflows and
          notifications

        ### Key response fields:

        A successful response returns a CustomerAlert object containing:

        - The threshold notification configuration with its unique ID and current status
        - The customer's evaluation status (ok, in_alarm, or evaluating)
        - Threshold notification metadata including type, threshold values, and update
          timestamps

        ### Usage guidelines:

        - Immediate evaluation: Set `evaluate_on_create` : `true` (default) for instant
          evaluation against existing customers
        - Uniqueness constraints: Each threshold notification must have a unique
          `uniqueness_key` within your organization. Use `release_uniqueness_key` :
          `true` when archiving to reuse keys
        - Notification type requirements: Different threshold notification types require
          specific fields (e.g., `billable_metric_id` for usage notifications,
          `credit_type_id` for credit-based threshold notifications)
        - Webhook delivery: Threshold notifications trigger webhook notifications for
          real-time integration with your systems. Configure webhook endpoints before
          creating threshold notifications
        - Performance at scale: Metronome's event-driven architecture processes
          threshold notification evaluations in real-time as usage events stream in,
          unlike competitors who rely on periodic polling or batch evaluation cycles

        Args:
          alert_type: Type of the threshold notification

          name: Name of the threshold notification

          threshold: Threshold value of the notification policy. Depending upon the notification
              type, this number may represent a financial amount, the days remaining, or a
              percentage reached.

          billable_metric_id: For threshold notifications of type `usage_threshold_reached`, specifies which
              billable metric to track the usage for.

          credit_grant_type_filters: An array of strings, representing a way to filter the credit grant this
              threshold notification applies to, by looking at the credit_grant_type field on
              the credit grant. This field is only defined for CreditPercentage and
              CreditBalance notifications

          credit_type_id: ID of the credit's currency, defaults to USD. If the specific notification type
              requires a pricing unit/currency, find the ID in the
              [Metronome app](https://app.metronome.com/offering/pricing-units).

          custom_field_filters: A list of custom field filters for threshold notification types that support
              advanced filtering. Only present for contract invoices.

          customer_id: If provided, will create this threshold notification for this specific customer.
              To create a notification for all customers, do not specify a `customer_id`.

          evaluate_on_create: If true, the threshold notification will evaluate immediately on customers that
              already meet the notification threshold. If false, it will only evaluate on
              future customers that trigger the threshold. Defaults to true.

          group_values: Only present for `spend_threshold_reached` notifications. Scope notification to
              a specific group key on individual line items.

          invoice_types_filter: Only supported for invoice_total_reached threshold notifications. A list of
              invoice types to evaluate.

          plan_id: If provided, will create this threshold notification for this specific plan. To
              create a notification for all customers, do not specify a `plan_id`.

          seat_filter: Required for `low_remaining_seat_balance_reached` notifications. The alert is
              scoped to this seat group key-value pair.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a record is made
              with a previously used uniqueness key, a new record will not be created and the
              request will fail with a 409 error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/alerts/create",
            body=maybe_transform(
                {
                    "alert_type": alert_type,
                    "name": name,
                    "threshold": threshold,
                    "billable_metric_id": billable_metric_id,
                    "credit_grant_type_filters": credit_grant_type_filters,
                    "credit_type_id": credit_type_id,
                    "custom_field_filters": custom_field_filters,
                    "customer_id": customer_id,
                    "evaluate_on_create": evaluate_on_create,
                    "group_values": group_values,
                    "invoice_types_filter": invoice_types_filter,
                    "plan_id": plan_id,
                    "seat_filter": seat_filter,
                    "uniqueness_key": uniqueness_key,
                },
                alert_create_params.AlertCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertCreateResponse,
        )

    def archive(
        self,
        *,
        id: str,
        release_uniqueness_key: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlertArchiveResponse:
        """
        Permanently disable a threshold notification and remove it from active
        monitoring across all customers. Archived threshold notifications stop
        evaluating immediately and can optionally release their uniqueness key for reuse
        in future threshold notification configurations.

        ### Use this endpoint to:

        - Decommission threshold notifications that are no longer needed
        - Clean up test or deprecated threshold notification configurations
        - Free up uniqueness keys for reuse with new threshold notifications
        - Stop threshold notification evaluations without losing historical
          configuration data
        - Disable outdated monitoring rules during pricing model transitions

        ### Key response fields:

        - data: Object containing the archived threshold notification's ID

        ### Usage guidelines:

        - Irreversible for evaluation: Archived threshold notifications cannot be
          re-enabled; create a new threshold notification to resume monitoring
        - Uniqueness key handling: Set `release_uniqueness_key` : `true` to reuse the
          key in future threshold notifications
        - Immediate effect: Threshold notification evaluation stops instantly across all
          customers
        - Historical preservation: Archive operation maintains threshold notification
          history and configuration for compliance and auditing

        Args:
          id: The Metronome ID of the threshold notification

          release_uniqueness_key: If true, resets the uniqueness key on this threshold notification so it can be
              re-used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/alerts/archive",
            body=maybe_transform(
                {
                    "id": id,
                    "release_uniqueness_key": release_uniqueness_key,
                },
                alert_archive_params.AlertArchiveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertArchiveResponse,
        )


class AsyncAlertsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAlertsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAlertsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAlertsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncAlertsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        alert_type: Literal[
            "low_credit_balance_reached",
            "spend_threshold_reached",
            "monthly_invoice_total_spend_threshold_reached",
            "low_remaining_days_in_plan_reached",
            "low_remaining_credit_percentage_reached",
            "usage_threshold_reached",
            "low_remaining_days_for_commit_segment_reached",
            "low_remaining_commit_balance_reached",
            "low_remaining_commit_percentage_reached",
            "low_remaining_days_for_contract_credit_segment_reached",
            "low_remaining_contract_credit_balance_reached",
            "low_remaining_contract_credit_percentage_reached",
            "low_remaining_contract_credit_and_commit_balance_reached",
            "invoice_total_reached",
            "low_remaining_seat_balance_reached",
        ],
        name: str,
        threshold: float,
        billable_metric_id: str | Omit = omit,
        credit_grant_type_filters: SequenceNotStr[str] | Omit = omit,
        credit_type_id: str | Omit = omit,
        custom_field_filters: Iterable[alert_create_params.CustomFieldFilter] | Omit = omit,
        customer_id: str | Omit = omit,
        evaluate_on_create: bool | Omit = omit,
        group_values: Iterable[alert_create_params.GroupValue] | Omit = omit,
        invoice_types_filter: SequenceNotStr[str] | Omit = omit,
        plan_id: str | Omit = omit,
        seat_filter: alert_create_params.SeatFilter | Omit = omit,
        uniqueness_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlertCreateResponse:
        """
        Create a new threshold notification to monitor customer spending, balances, and
        billing metrics in real-time. Metronome's notification system provides
        industry-leading speed with immediate evaluation capabilities, enabling you to
        proactively manage customer accounts and prevent revenue leakage.

        This endpoint creates configurable threshold notifications that continuously
        monitor various billing thresholds including spend limits, credit balances,
        commitment utilization, and invoice totals. Threshold notifications can be
        configured globally for all customers or targeted to specific customer accounts.

        ### Use this endpoint to:

        - Proactively monitor customer spending patterns to prevent unexpected overages
          or credit exhaustion
        - Automate notifications when customers approach commitment limits or credit
          thresholds
        - Enable real-time intervention for accounts at risk of churn or payment issues
        - Scale billing operations by automating threshold-based workflows and
          notifications

        ### Key response fields:

        A successful response returns a CustomerAlert object containing:

        - The threshold notification configuration with its unique ID and current status
        - The customer's evaluation status (ok, in_alarm, or evaluating)
        - Threshold notification metadata including type, threshold values, and update
          timestamps

        ### Usage guidelines:

        - Immediate evaluation: Set `evaluate_on_create` : `true` (default) for instant
          evaluation against existing customers
        - Uniqueness constraints: Each threshold notification must have a unique
          `uniqueness_key` within your organization. Use `release_uniqueness_key` :
          `true` when archiving to reuse keys
        - Notification type requirements: Different threshold notification types require
          specific fields (e.g., `billable_metric_id` for usage notifications,
          `credit_type_id` for credit-based threshold notifications)
        - Webhook delivery: Threshold notifications trigger webhook notifications for
          real-time integration with your systems. Configure webhook endpoints before
          creating threshold notifications
        - Performance at scale: Metronome's event-driven architecture processes
          threshold notification evaluations in real-time as usage events stream in,
          unlike competitors who rely on periodic polling or batch evaluation cycles

        Args:
          alert_type: Type of the threshold notification

          name: Name of the threshold notification

          threshold: Threshold value of the notification policy. Depending upon the notification
              type, this number may represent a financial amount, the days remaining, or a
              percentage reached.

          billable_metric_id: For threshold notifications of type `usage_threshold_reached`, specifies which
              billable metric to track the usage for.

          credit_grant_type_filters: An array of strings, representing a way to filter the credit grant this
              threshold notification applies to, by looking at the credit_grant_type field on
              the credit grant. This field is only defined for CreditPercentage and
              CreditBalance notifications

          credit_type_id: ID of the credit's currency, defaults to USD. If the specific notification type
              requires a pricing unit/currency, find the ID in the
              [Metronome app](https://app.metronome.com/offering/pricing-units).

          custom_field_filters: A list of custom field filters for threshold notification types that support
              advanced filtering. Only present for contract invoices.

          customer_id: If provided, will create this threshold notification for this specific customer.
              To create a notification for all customers, do not specify a `customer_id`.

          evaluate_on_create: If true, the threshold notification will evaluate immediately on customers that
              already meet the notification threshold. If false, it will only evaluate on
              future customers that trigger the threshold. Defaults to true.

          group_values: Only present for `spend_threshold_reached` notifications. Scope notification to
              a specific group key on individual line items.

          invoice_types_filter: Only supported for invoice_total_reached threshold notifications. A list of
              invoice types to evaluate.

          plan_id: If provided, will create this threshold notification for this specific plan. To
              create a notification for all customers, do not specify a `plan_id`.

          seat_filter: Required for `low_remaining_seat_balance_reached` notifications. The alert is
              scoped to this seat group key-value pair.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a record is made
              with a previously used uniqueness key, a new record will not be created and the
              request will fail with a 409 error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/alerts/create",
            body=await async_maybe_transform(
                {
                    "alert_type": alert_type,
                    "name": name,
                    "threshold": threshold,
                    "billable_metric_id": billable_metric_id,
                    "credit_grant_type_filters": credit_grant_type_filters,
                    "credit_type_id": credit_type_id,
                    "custom_field_filters": custom_field_filters,
                    "customer_id": customer_id,
                    "evaluate_on_create": evaluate_on_create,
                    "group_values": group_values,
                    "invoice_types_filter": invoice_types_filter,
                    "plan_id": plan_id,
                    "seat_filter": seat_filter,
                    "uniqueness_key": uniqueness_key,
                },
                alert_create_params.AlertCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertCreateResponse,
        )

    async def archive(
        self,
        *,
        id: str,
        release_uniqueness_key: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlertArchiveResponse:
        """
        Permanently disable a threshold notification and remove it from active
        monitoring across all customers. Archived threshold notifications stop
        evaluating immediately and can optionally release their uniqueness key for reuse
        in future threshold notification configurations.

        ### Use this endpoint to:

        - Decommission threshold notifications that are no longer needed
        - Clean up test or deprecated threshold notification configurations
        - Free up uniqueness keys for reuse with new threshold notifications
        - Stop threshold notification evaluations without losing historical
          configuration data
        - Disable outdated monitoring rules during pricing model transitions

        ### Key response fields:

        - data: Object containing the archived threshold notification's ID

        ### Usage guidelines:

        - Irreversible for evaluation: Archived threshold notifications cannot be
          re-enabled; create a new threshold notification to resume monitoring
        - Uniqueness key handling: Set `release_uniqueness_key` : `true` to reuse the
          key in future threshold notifications
        - Immediate effect: Threshold notification evaluation stops instantly across all
          customers
        - Historical preservation: Archive operation maintains threshold notification
          history and configuration for compliance and auditing

        Args:
          id: The Metronome ID of the threshold notification

          release_uniqueness_key: If true, resets the uniqueness key on this threshold notification so it can be
              re-used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/alerts/archive",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "release_uniqueness_key": release_uniqueness_key,
                },
                alert_archive_params.AlertArchiveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertArchiveResponse,
        )


class AlertsResourceWithRawResponse:
    def __init__(self, alerts: AlertsResource) -> None:
        self._alerts = alerts

        self.create = to_raw_response_wrapper(
            alerts.create,
        )
        self.archive = to_raw_response_wrapper(
            alerts.archive,
        )


class AsyncAlertsResourceWithRawResponse:
    def __init__(self, alerts: AsyncAlertsResource) -> None:
        self._alerts = alerts

        self.create = async_to_raw_response_wrapper(
            alerts.create,
        )
        self.archive = async_to_raw_response_wrapper(
            alerts.archive,
        )


class AlertsResourceWithStreamingResponse:
    def __init__(self, alerts: AlertsResource) -> None:
        self._alerts = alerts

        self.create = to_streamed_response_wrapper(
            alerts.create,
        )
        self.archive = to_streamed_response_wrapper(
            alerts.archive,
        )


class AsyncAlertsResourceWithStreamingResponse:
    def __init__(self, alerts: AsyncAlertsResource) -> None:
        self._alerts = alerts

        self.create = async_to_streamed_response_wrapper(
            alerts.create,
        )
        self.archive = async_to_streamed_response_wrapper(
            alerts.archive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["QuantityConversion"]


class QuantityConversion(BaseModel):
    """Optional.

    Only valid for USAGE products. If provided, the quantity will be converted using the provided conversion factor and operation. For example, if the operation is "multiply" and the conversion factor is 100, then the quantity will be multiplied by 100. This can be used in cases where data is sent in one unit and priced in another.  For example, data could be sent in MB and priced in GB. In this case, the conversion factor would be 1024 and the operation would be "divide".
    """

    conversion_factor: float
    """The factor to multiply or divide the quantity by."""

    operation: Literal["MULTIPLY", "DIVIDE"]
    """The operation to perform on the quantity"""

    name: Optional[str] = None
    """Optional name for this conversion."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CommitListParams"]


class CommitListParams(TypedDict, total=False):
    customer_id: Required[str]

    commit_id: str

    covering_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Include only commits that have access schedules that "cover" the provided date"""

    effective_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Include only commits that have any access before the provided date (exclusive)"""

    include_archived: bool
    """Include archived commits and commits from archived contracts."""

    include_balance: bool
    """Include the balance in the response.

    Setting this flag may cause the query to be slower.
    """

    include_contract_commits: bool
    """Include commits on the contract level."""

    include_ledgers: bool
    """Include commit ledgers in the response.

    Setting this flag may cause the query to be slower.
    """

    limit: int
    """The maximum number of commits to return. Defaults to 25."""

    next_page: str
    """The next page token from a previous response."""

    starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Include only commits that have any access on or after the provided date"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContractScheduleProServicesInvoiceParams", "LineItem"]


class ContractScheduleProServicesInvoiceParams(TypedDict, total=False):
    contract_id: Required[str]

    customer_id: Required[str]

    issued_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The date the invoice is issued"""

    line_items: Required[Iterable[LineItem]]
    """Each line requires an amount or both unit_price and quantity."""

    netsuite_invoice_header_end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The end date of the invoice header in Netsuite"""

    netsuite_invoice_header_start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The start date of the invoice header in Netsuite"""


class LineItem(TypedDict, total=False):
    """Describes the line item for a professional service charge on an invoice."""

    professional_service_id: Required[str]

    amendment_id: str
    """If the professional_service_id was added on an amendment, this is required."""

    amount: float
    """Amount for the term on the new invoice."""

    metadata: str
    """For client use."""

    netsuite_invoice_billing_end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The end date for the billing period on the invoice."""

    netsuite_invoice_billing_start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The start date for the billing period on the invoice."""

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount.
    """

    unit_price: float
    """If specified, this overrides the unit price on the pro service term.

    Must also provide quantity (but not amount) if providing unit_price.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContractAddManualBalanceEntryParams"]


class ContractAddManualBalanceEntryParams(TypedDict, total=False):
    id: Required[str]
    """ID of the balance (commit or credit) to update."""

    amount: Required[float]
    """Amount to add to the segment.

    A negative number will draw down from the balance.
    """

    customer_id: Required[str]
    """ID of the customer whose balance is to be updated."""

    reason: Required[str]
    """Reason for the manual adjustment. This will be displayed in the ledger."""

    segment_id: Required[str]
    """ID of the segment to update."""

    contract_id: str
    """ID of the contract to update. Leave blank to update a customer level balance."""

    per_group_amounts: Dict[str, float]
    """
    If using individually configured commits/credits attached to seat managed
    subscriptions, the amount to add for each seat. Must sum to total amount.
    """

    timestamp: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """RFC 3339 timestamp indicating when the manual adjustment takes place.

    If not provided, it will default to the start of the segment.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime

import httpx

from .rates import (
    RatesResource,
    AsyncRatesResource,
    RatesResourceWithRawResponse,
    AsyncRatesResourceWithRawResponse,
    RatesResourceWithStreamingResponse,
    AsyncRatesResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncCursorPage, AsyncCursorPage
from .product_orders import (
    ProductOrdersResource,
    AsyncProductOrdersResource,
    ProductOrdersResourceWithRawResponse,
    AsyncProductOrdersResourceWithRawResponse,
    ProductOrdersResourceWithStreamingResponse,
    AsyncProductOrdersResourceWithStreamingResponse,
)
from .named_schedules import (
    NamedSchedulesResource,
    AsyncNamedSchedulesResource,
    NamedSchedulesResourceWithRawResponse,
    AsyncNamedSchedulesResourceWithRawResponse,
    NamedSchedulesResourceWithStreamingResponse,
    AsyncNamedSchedulesResourceWithStreamingResponse,
)
from ....._base_client import AsyncPaginator, make_request_options
from .....types.v1.contracts import (
    rate_card_list_params,
    rate_card_create_params,
    rate_card_update_params,
    rate_card_archive_params,
    rate_card_retrieve_params,
    rate_card_retrieve_rate_schedule_params,
)
from .....types.v1.contracts.rate_card_list_response import RateCardListResponse
from .....types.v1.contracts.rate_card_create_response import RateCardCreateResponse
from .....types.v1.contracts.rate_card_update_response import RateCardUpdateResponse
from .....types.v1.contracts.rate_card_archive_response import RateCardArchiveResponse
from .....types.v1.contracts.rate_card_retrieve_response import RateCardRetrieveResponse
from .....types.v1.contracts.rate_card_retrieve_rate_schedule_response import RateCardRetrieveRateScheduleResponse

__all__ = ["RateCardsResource", "AsyncRateCardsResource"]


class RateCardsResource(SyncAPIResource):
    @cached_property
    def product_orders(self) -> ProductOrdersResource:
        return ProductOrdersResource(self._client)

    @cached_property
    def rates(self) -> RatesResource:
        return RatesResource(self._client)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResource:
        return NamedSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> RateCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return RateCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RateCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return RateCardsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        aliases: Iterable[rate_card_create_params.Alias] | Omit = omit,
        credit_type_conversions: Iterable[rate_card_create_params.CreditTypeConversion] | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        description: str | Omit = omit,
        fiat_credit_type_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardCreateResponse:
        """In Metronome, the rate card is the central location for your pricing.

        Rate cards
        were built with new product launches and pricing changes in mind - you can
        update your products and pricing in one place, and that change will be
        automatically propagated across your customer cohorts. Most clients need only
        maintain one or a few rate cards within Metronome.

        ### Use this endpoint to:

        - Create a rate card with a name and description
        - Define the rate card's single underlying fiat currency, and any number of
          conversion rates between that fiat currency and custom pricing units. You can
          then add products and associated rates in the fiat currency or custom pricing
          unit for which you have defined a conversion rate.
        - Set aliases for the rate card. Aliases are human-readable names that you can
          use in the place of the id of the rate card when provisioning a customer's
          contract. By using an alias, you can easily create a contract and provision a
          customer by choosing the paygo rate card, without storing the rate card id in
          your internal systems. This is helpful when launching a new rate card for
          paygo customers, you can update the alias for paygo to be scheduled to be
          assigned to the new rate card without updating your code.

        ### Key response fields:

        - The ID of the rate card you just created

        ### Usage guidelines:

        - After creating a rate card, you can now use the addRate or addRates endpoints
          to add products and their prices to it
        - A rate card alias can only be used by one rate card at a time. If you create a
          contract with a rate card alias that is already in use by another rate card,
          the original rate card's alias schedule will be updated. The alias will
          reference the rate card to which it was most recently assigned.

        Args:
          name: Used only in UI/API. It is not exposed to end customers.

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          credit_type_conversions: Required when using custom pricing units in rates.

          custom_fields: Custom fields to be added eg. { "key1": "value1", "key2": "value2" }

          fiat_credit_type_id: The Metronome ID of the credit type to associate with the rate card, defaults to
              USD (cents) if not passed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/create",
            body=maybe_transform(
                {
                    "name": name,
                    "aliases": aliases,
                    "credit_type_conversions": credit_type_conversions,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fiat_credit_type_id": fiat_credit_type_id,
                },
                rate_card_create_params.RateCardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardCreateResponse,
        )

    def retrieve(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardRetrieveResponse:
        """Return details for a specific rate card including name, description, and
        aliases.

        This endpoint does not return rates - use the dedicated getRates or
        getRateSchedule endpoints to understand the rates on a rate card.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/get",
            body=maybe_transform({"id": id}, rate_card_retrieve_params.RateCardRetrieveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardRetrieveResponse,
        )

    def update(
        self,
        *,
        rate_card_id: str,
        aliases: Iterable[rate_card_update_params.Alias] | Omit = omit,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardUpdateResponse:
        """
        Update the metadata properties of an existing rate card, including its name,
        description, and aliases. This endpoint is designed for managing rate card
        identity and reference aliases rather than modifying pricing rates.

        Modifies the descriptive properties and alias configuration of a rate card
        without affecting the underlying pricing rates or schedules. This allows you to
        update how a rate card is identified and referenced throughout your system.

        ### Use this endpoint to:

        - Rate card renaming: Update display names or descriptions for organizational
          clarity
        - Alias management: Add, modify, or schedule alias transitions for seamless rate
          card migrations
        - Documentation updates: Keep rate card descriptions current with business
          context
        - Self-serve provisioning setup: Configure aliases to enable code-free rate card
          transitions

        #### Active contract impact:

        - Alias changes: Already-created contracts continue using their originally
          assigned rate cards.
        - Other changes made using this endpoint will only impact the Metronome UI.

        #### Grandfathering existing PLG customer pricing:

        - Rate card aliases support scheduled transitions, enabling seamless rate card
          migrations for new customers, allowing existing customers to be grandfathered
          into their existing prices without code. Note that there are multiple
          mechanisms to support grandfathering in Metronome.

        #### How scheduled aliases work for PLG grandfathering:

        Initial setup:

        - Add alias to current rate card: Assign a stable alias (e.g.,
          "standard-pricing") to your active rate card
        - Reference alias during contract creation: Configure your self-serve workflow
          to create contracts using `rate_card_alias` instead of direct `rate_card_id`
        - Automatic resolution: New contracts referencing the alias automatically
          resolve to the rate card associated with the alias at the point in time of
          provisioning

        #### Grandfathering process:

        - Create new rate card: Build your new rate card with updated pricing structure
        - Schedule alias transition: Add the same alias to the new rate card with a
          `starting_at` timestamp
        - Automatic cutover: Starting at the scheduled time, new contracts created in
          your PLG workflow using that alias will automatically reference the new rate
          card

        Args:
          rate_card_id: ID of the rate card to update

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          name: Used only in UI/API. It is not exposed to end customers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/update",
            body=maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "aliases": aliases,
                    "description": description,
                    "name": name,
                },
                rate_card_update_params.RateCardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardUpdateResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        body: object | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[RateCardListResponse]:
        """List all rate cards.

        Returns rate card IDs, names, descriptions, aliases, and
        other details. To view the rates associated with a given rate card, use the
        getRates or getRateSchedule endpoints.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contract-pricing/rate-cards/list",
            page=SyncCursorPage[RateCardListResponse],
            body=maybe_transform(body, rate_card_list_params.RateCardListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_list_params.RateCardListParams,
                ),
            ),
            model=RateCardListResponse,
            method="post",
        )

    def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardArchiveResponse:
        """
        Permanently disable a rate card by archiving it, preventing use in new contracts
        while preserving existing contract pricing. Use this when retiring old pricing
        models, consolidating rate cards, or removing outdated pricing structures.
        Returns the archived rate card ID and stops the rate card from appearing in
        contract creation workflows.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/archive",
            body=maybe_transform({"id": id}, rate_card_archive_params.RateCardArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardArchiveResponse,
        )

    def retrieve_rate_schedule(
        self,
        *,
        rate_card_id: str,
        starting_at: Union[str, datetime],
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        ending_before: Union[str, datetime] | Omit = omit,
        selectors: Iterable[rate_card_retrieve_rate_schedule_params.Selector] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardRetrieveRateScheduleResponse:
        """A rate card defines the prices that you charge for your products.

        Rate cards
        support scheduled changes over time, to allow you to easily roll out pricing
        changes and new product launches across your customer base. Use this endpoint to
        understand the rate schedule `starting_at` a given date, optionally filtering
        the list of rates returned based on product id or pricing group values. For
        example, you may want to display a schedule of upcoming price changes for a
        given product in your product experience - use this endpoint to fetch that
        information from its source of truth in Metronome.

        If you want to understand the rates for a specific customer's contract,
        inclusive of contract-level overrides, use the `getContractRateSchedule`
        endpoint.

        Args:
          rate_card_id: ID of the rate card to get the schedule for

          starting_at: inclusive starting point for the rates schedule

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          ending_before: optional exclusive end date for the rates schedule. When not specified rates
              will show all future schedule segments.

          selectors: List of rate selectors, rates matching ANY of the selector will be included in
              the response Passing no selectors will result in all rates being returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/getRateSchedule",
            body=maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "starting_at": starting_at,
                    "ending_before": ending_before,
                    "selectors": selectors,
                },
                rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
                ),
            ),
            cast_to=RateCardRetrieveRateScheduleResponse,
        )


class AsyncRateCardsResource(AsyncAPIResource):
    @cached_property
    def product_orders(self) -> AsyncProductOrdersResource:
        return AsyncProductOrdersResource(self._client)

    @cached_property
    def rates(self) -> AsyncRatesResource:
        return AsyncRatesResource(self._client)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResource:
        return AsyncNamedSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRateCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRateCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRateCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncRateCardsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        aliases: Iterable[rate_card_create_params.Alias] | Omit = omit,
        credit_type_conversions: Iterable[rate_card_create_params.CreditTypeConversion] | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        description: str | Omit = omit,
        fiat_credit_type_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardCreateResponse:
        """In Metronome, the rate card is the central location for your pricing.

        Rate cards
        were built with new product launches and pricing changes in mind - you can
        update your products and pricing in one place, and that change will be
        automatically propagated across your customer cohorts. Most clients need only
        maintain one or a few rate cards within Metronome.

        ### Use this endpoint to:

        - Create a rate card with a name and description
        - Define the rate card's single underlying fiat currency, and any number of
          conversion rates between that fiat currency and custom pricing units. You can
          then add products and associated rates in the fiat currency or custom pricing
          unit for which you have defined a conversion rate.
        - Set aliases for the rate card. Aliases are human-readable names that you can
          use in the place of the id of the rate card when provisioning a customer's
          contract. By using an alias, you can easily create a contract and provision a
          customer by choosing the paygo rate card, without storing the rate card id in
          your internal systems. This is helpful when launching a new rate card for
          paygo customers, you can update the alias for paygo to be scheduled to be
          assigned to the new rate card without updating your code.

        ### Key response fields:

        - The ID of the rate card you just created

        ### Usage guidelines:

        - After creating a rate card, you can now use the addRate or addRates endpoints
          to add products and their prices to it
        - A rate card alias can only be used by one rate card at a time. If you create a
          contract with a rate card alias that is already in use by another rate card,
          the original rate card's alias schedule will be updated. The alias will
          reference the rate card to which it was most recently assigned.

        Args:
          name: Used only in UI/API. It is not exposed to end customers.

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          credit_type_conversions: Required when using custom pricing units in rates.

          custom_fields: Custom fields to be added eg. { "key1": "value1", "key2": "value2" }

          fiat_credit_type_id: The Metronome ID of the credit type to associate with the rate card, defaults to
              USD (cents) if not passed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/create",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "aliases": aliases,
                    "credit_type_conversions": credit_type_conversions,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fiat_credit_type_id": fiat_credit_type_id,
                },
                rate_card_create_params.RateCardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardCreateResponse,
        )

    async def retrieve(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardRetrieveResponse:
        """Return details for a specific rate card including name, description, and
        aliases.

        This endpoint does not return rates - use the dedicated getRates or
        getRateSchedule endpoints to understand the rates on a rate card.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/get",
            body=await async_maybe_transform({"id": id}, rate_card_retrieve_params.RateCardRetrieveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardRetrieveResponse,
        )

    async def update(
        self,
        *,
        rate_card_id: str,
        aliases: Iterable[rate_card_update_params.Alias] | Omit = omit,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardUpdateResponse:
        """
        Update the metadata properties of an existing rate card, including its name,
        description, and aliases. This endpoint is designed for managing rate card
        identity and reference aliases rather than modifying pricing rates.

        Modifies the descriptive properties and alias configuration of a rate card
        without affecting the underlying pricing rates or schedules. This allows you to
        update how a rate card is identified and referenced throughout your system.

        ### Use this endpoint to:

        - Rate card renaming: Update display names or descriptions for organizational
          clarity
        - Alias management: Add, modify, or schedule alias transitions for seamless rate
          card migrations
        - Documentation updates: Keep rate card descriptions current with business
          context
        - Self-serve provisioning setup: Configure aliases to enable code-free rate card
          transitions

        #### Active contract impact:

        - Alias changes: Already-created contracts continue using their originally
          assigned rate cards.
        - Other changes made using this endpoint will only impact the Metronome UI.

        #### Grandfathering existing PLG customer pricing:

        - Rate card aliases support scheduled transitions, enabling seamless rate card
          migrations for new customers, allowing existing customers to be grandfathered
          into their existing prices without code. Note that there are multiple
          mechanisms to support grandfathering in Metronome.

        #### How scheduled aliases work for PLG grandfathering:

        Initial setup:

        - Add alias to current rate card: Assign a stable alias (e.g.,
          "standard-pricing") to your active rate card
        - Reference alias during contract creation: Configure your self-serve workflow
          to create contracts using `rate_card_alias` instead of direct `rate_card_id`
        - Automatic resolution: New contracts referencing the alias automatically
          resolve to the rate card associated with the alias at the point in time of
          provisioning

        #### Grandfathering process:

        - Create new rate card: Build your new rate card with updated pricing structure
        - Schedule alias transition: Add the same alias to the new rate card with a
          `starting_at` timestamp
        - Automatic cutover: Starting at the scheduled time, new contracts created in
          your PLG workflow using that alias will automatically reference the new rate
          card

        Args:
          rate_card_id: ID of the rate card to update

          aliases: Reference this alias when creating a contract. If the same alias is assigned to
              multiple rate cards, it will reference the rate card to which it was most
              recently assigned. It is not exposed to end customers.

          name: Used only in UI/API. It is not exposed to end customers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/update",
            body=await async_maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "aliases": aliases,
                    "description": description,
                    "name": name,
                },
                rate_card_update_params.RateCardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardUpdateResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        body: object | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RateCardListResponse, AsyncCursorPage[RateCardListResponse]]:
        """List all rate cards.

        Returns rate card IDs, names, descriptions, aliases, and
        other details. To view the rates associated with a given rate card, use the
        getRates or getRateSchedule endpoints.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contract-pricing/rate-cards/list",
            page=AsyncCursorPage[RateCardListResponse],
            body=maybe_transform(body, rate_card_list_params.RateCardListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_list_params.RateCardListParams,
                ),
            ),
            model=RateCardListResponse,
            method="post",
        )

    async def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardArchiveResponse:
        """
        Permanently disable a rate card by archiving it, preventing use in new contracts
        while preserving existing contract pricing. Use this when retiring old pricing
        models, consolidating rate cards, or removing outdated pricing structures.
        Returns the archived rate card ID and stops the rate card from appearing in
        contract creation workflows.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/archive",
            body=await async_maybe_transform({"id": id}, rate_card_archive_params.RateCardArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardArchiveResponse,
        )

    async def retrieve_rate_schedule(
        self,
        *,
        rate_card_id: str,
        starting_at: Union[str, datetime],
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        ending_before: Union[str, datetime] | Omit = omit,
        selectors: Iterable[rate_card_retrieve_rate_schedule_params.Selector] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardRetrieveRateScheduleResponse:
        """A rate card defines the prices that you charge for your products.

        Rate cards
        support scheduled changes over time, to allow you to easily roll out pricing
        changes and new product launches across your customer base. Use this endpoint to
        understand the rate schedule `starting_at` a given date, optionally filtering
        the list of rates returned based on product id or pricing group values. For
        example, you may want to display a schedule of upcoming price changes for a
        given product in your product experience - use this endpoint to fetch that
        information from its source of truth in Metronome.

        If you want to understand the rates for a specific customer's contract,
        inclusive of contract-level overrides, use the `getContractRateSchedule`
        endpoint.

        Args:
          rate_card_id: ID of the rate card to get the schedule for

          starting_at: inclusive starting point for the rates schedule

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          ending_before: optional exclusive end date for the rates schedule. When not specified rates
              will show all future schedule segments.

          selectors: List of rate selectors, rates matching ANY of the selector will be included in
              the response Passing no selectors will result in all rates being returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/getRateSchedule",
            body=await async_maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "starting_at": starting_at,
                    "ending_before": ending_before,
                    "selectors": selectors,
                },
                rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    rate_card_retrieve_rate_schedule_params.RateCardRetrieveRateScheduleParams,
                ),
            ),
            cast_to=RateCardRetrieveRateScheduleResponse,
        )


class RateCardsResourceWithRawResponse:
    def __init__(self, rate_cards: RateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = to_raw_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = to_raw_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = to_raw_response_wrapper(
            rate_cards.update,
        )
        self.list = to_raw_response_wrapper(
            rate_cards.list,
        )
        self.archive = to_raw_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = to_raw_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> ProductOrdersResourceWithRawResponse:
        return ProductOrdersResourceWithRawResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> RatesResourceWithRawResponse:
        return RatesResourceWithRawResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResourceWithRawResponse:
        return NamedSchedulesResourceWithRawResponse(self._rate_cards.named_schedules)


class AsyncRateCardsResourceWithRawResponse:
    def __init__(self, rate_cards: AsyncRateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = async_to_raw_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            rate_cards.update,
        )
        self.list = async_to_raw_response_wrapper(
            rate_cards.list,
        )
        self.archive = async_to_raw_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = async_to_raw_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> AsyncProductOrdersResourceWithRawResponse:
        return AsyncProductOrdersResourceWithRawResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> AsyncRatesResourceWithRawResponse:
        return AsyncRatesResourceWithRawResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResourceWithRawResponse:
        return AsyncNamedSchedulesResourceWithRawResponse(self._rate_cards.named_schedules)


class RateCardsResourceWithStreamingResponse:
    def __init__(self, rate_cards: RateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = to_streamed_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            rate_cards.update,
        )
        self.list = to_streamed_response_wrapper(
            rate_cards.list,
        )
        self.archive = to_streamed_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = to_streamed_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> ProductOrdersResourceWithStreamingResponse:
        return ProductOrdersResourceWithStreamingResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> RatesResourceWithStreamingResponse:
        return RatesResourceWithStreamingResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> NamedSchedulesResourceWithStreamingResponse:
        return NamedSchedulesResourceWithStreamingResponse(self._rate_cards.named_schedules)


class AsyncRateCardsResourceWithStreamingResponse:
    def __init__(self, rate_cards: AsyncRateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = async_to_streamed_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            rate_cards.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            rate_cards.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rate_cards.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            rate_cards.archive,
        )
        self.retrieve_rate_schedule = async_to_streamed_response_wrapper(
            rate_cards.retrieve_rate_schedule,
        )

    @cached_property
    def product_orders(self) -> AsyncProductOrdersResourceWithStreamingResponse:
        return AsyncProductOrdersResourceWithStreamingResponse(self._rate_cards.product_orders)

    @cached_property
    def rates(self) -> AsyncRatesResourceWithStreamingResponse:
        return AsyncRatesResourceWithStreamingResponse(self._rate_cards.rates)

    @cached_property
    def named_schedules(self) -> AsyncNamedSchedulesResourceWithStreamingResponse:
        return AsyncNamedSchedulesResourceWithStreamingResponse(self._rate_cards.named_schedules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from ..._models import BaseModel

__all__ = ["CommitSpecifierInput"]


class CommitSpecifierInput(BaseModel):
    presentation_group_values: Optional[Dict[str, str]] = None

    pricing_group_values: Optional[Dict[str, str]] = None

    product_id: Optional[str] = None
    """
    If provided, the specifier will only apply to the product with the specified ID.
    """

    product_tags: Optional[List[str]] = None
    """
    If provided, the specifier will only apply to products with all the specified
    tags.
    """
